package org.bench4q.agent.scenario.engine;

import java.util.Date;
import java.util.UUID;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ThreadPoolExecutor.DiscardPolicy;

import org.apache.log4j.Logger;
import org.bench4q.agent.datacollector.DataCollector;
import org.bench4q.agent.datacollector.impl.ScenarioResultCollector;
import org.bench4q.agent.plugin.PluginManager;
import org.bench4q.agent.scenario.Scenario;

public class ScenarioContext {
	private static final long keepAliveTime = 10;
	private UUID testId;
	private Date startDate;
	private Date endDate;
	private ThreadPoolExecutor executor;
	private Scenario scenario;
	private boolean finished;
	private DataCollector dataStatistics;
	private PluginManager pluginManager;

	public UUID getTestId() {
		return testId;
	}

	private void setTestId(UUID testId) {
		this.testId = testId;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date saveStartDate) {
		this.startDate = saveStartDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public ThreadPoolExecutor getExecutor() {
		return executor;
	}

	public void setExecutorService(ThreadPoolExecutor executor) {
		this.executor = executor;
	}

	public Scenario getScenario() {
		return scenario;
	}

	public void setScenario(Scenario scenario) {
		this.scenario = scenario;
	}

	public boolean isFinished() {
		return finished;
	}

	public void setFinished(boolean finished) {
		this.finished = finished;
	}

	public DataCollector getDataStatistics() {
		return dataStatistics;
	}

	private void setDataStatistics(DataCollector dataStatistics) {
		this.dataStatistics = dataStatistics;
	}

	private PluginManager getPluginManager() {
		return pluginManager;
	}

	private void setPluginManager(PluginManager pluginManager) {
		this.pluginManager = pluginManager;
	}

	private ScenarioContext() {
	}

	public static ScenarioContext buildScenarioContext(UUID testId,
			final Scenario scenario, int poolSize, PluginManager pluginManager) {
		ScenarioContext scenarioContext = buildScenarioContextWithoutScenario(
				testId, poolSize, pluginManager);
		scenarioContext.setScenario(scenario);
		return scenarioContext;
	}

	public static ScenarioContext buildScenarioContextWithoutScenario(
			UUID testId, int poolSize, PluginManager pluginManager) {
		ScenarioContext scenarioContext = new ScenarioContext();
		scenarioContext.setTestId(testId);
		scenarioContext.setPluginManager(pluginManager);
		final ArrayBlockingQueue<Runnable> workQueue = new ArrayBlockingQueue<Runnable>(
				poolSize);
		ThreadPoolExecutor executor = new ThreadPoolExecutor(poolSize,
				poolSize, keepAliveTime, TimeUnit.MINUTES, workQueue,
				new DiscardPolicy());
		scenarioContext.setStartDate(new Date(System.currentTimeMillis()));
		scenarioContext.setExecutorService(executor);
		scenarioContext.setDataStatistics(new ScenarioResultCollector(testId));
		return scenarioContext;
	}

	public ScenarioContext addScenrio(Scenario scenario) {
		this.setScenario(scenario);
		return this;
	}

	/**
	 * Now, I tolerate that if the requiredLoad <
	 * this.getExecutor.getCorePoolSize(), then the excess threads will be
	 * killed when its current task complete
	 * 
	 * @param requiredLoad
	 */
	void updatePopulation(int requiredLoad) {
		this.getExecutor().setCorePoolSize(requiredLoad);
		this.getExecutor().setMaximumPoolSize(requiredLoad);
	}

	public void addTask() {
		if (this.isFinished()) {
			return;
		}
		this.getExecutor().execute(new VUser(this, 1, getPluginManager()));
		Logger.getLogger(this.getClass()).info(
				this.getExecutor().getActiveCount());
	}

	public void initTasks() {
		for (int i = 0; i < this.getExecutor().getCorePoolSize(); i++) {
			addTask();
		}
	}
}
