package org.bench4q.agent.scenario.engine;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import org.apache.log4j.Logger;
import org.bench4q.agent.plugin.PluginManager;
import org.bench4q.agent.scenario.Scenario;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class ScenarioEngine {
	private Map<UUID, ScenarioContext> runningTests;
	private Logger logger = Logger.getLogger(ScenarioEngine.class);
	private PluginManager pluginManager;

	public ScenarioEngine() {
		this.setRunningTests(new HashMap<UUID, ScenarioContext>());
	}

	public Map<UUID, ScenarioContext> getRunningTests() {
		return runningTests;
	}

	private void setRunningTests(Map<UUID, ScenarioContext> runningTests) {
		this.runningTests = runningTests;
	}

	private PluginManager getPluginManager() {
		return pluginManager;
	}

	@Autowired
	private void setPluginManager(PluginManager pluginManager) {
		this.pluginManager = pluginManager;
	}

	public void addRunningTestWithoutScenario(UUID runId, int poolSize) {
		try {
			final ScenarioContext scenarioContext = ScenarioContext
					.buildScenarioContextWithoutScenario(runId, poolSize,
							getPluginManager());
			this.getRunningTests().put(runId, scenarioContext);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void submitScenario(UUID runId, final Scenario scenario) {
		try {
			this.getRunningTests().get(runId).addScenrio(scenario);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean runWith(UUID runId) {
		if (!this.getRunningTests().containsKey(runId)) {
			return false;
		}
		final ScenarioContext context = this.getRunningTests().get(runId);
		return runWith(context);
	}

	private boolean runWith(final ScenarioContext scenarioContext) {
		if (scenarioContext == null) {
			logger.error("The context required is null");
			return false;
		}
		scenarioContext.initTasks();
		return true;
	}

	public void updatePopulation(UUID testId, int requiredLoad) {
		ScenarioContext context = this.getRunningTests().get(testId);
		context.updatePopulation(requiredLoad);
	}
}
