package org.bench4q.master.infrastructure.communication.impl;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.bench4q.master.domain.entity.Agent;
import org.bench4q.master.infrastructure.communication.AgentMessenger;
import org.bench4q.master.infrastructure.faultolerence.FaultTolerenceFactory;
import org.bench4q.share.communication.HttpRequester;
import org.bench4q.share.communication.HttpRequester.HttpResponse;
import org.bench4q.share.helper.MarshalHelper;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.agent.RunScenarioResultModel;
import org.bench4q.share.models.agent.ServerStatusModel;
import org.bench4q.share.models.agent.StopTestModel;
import org.bench4q.share.models.agent.TestBriefStatusModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class AgentMessengerImpl implements AgentMessenger {
	private HttpRequester httpRequester;
	private Logger logger = Logger.getLogger(AgentMessengerImpl.class);

	private HttpRequester getHttpRequester() {
		return httpRequester;
	}

	@Autowired
	private void setHttpRequester(HttpRequester httpRequester) {
		this.httpRequester = httpRequester;
	}

	private void logIt(HttpResponse httpResponse, Throwable e) {
		logger.error(HttpRequester.isInvalidResponse(httpResponse) ? ""
				: httpResponse.getContent(), e);
	}

	public RunScenarioResultModel bookTest(Agent agent, int requireLoad) {
		HttpResponse httpResponse = null;
		try {
			httpResponse = this.getHttpRequester().sendGet(
					buildBaseUrl(agent) + "/test/bookTest/" + requireLoad,
					null, null);
			System.out.println(buildBaseUrl(agent) + "/test/bookTest/"
					+ requireLoad);
			return (RunScenarioResultModel) MarshalHelper.unmarshal(
					RunScenarioResultModel.class, httpResponse.getContent());
		} catch (Exception e) {
			logger.error(httpResponse.getContent(), e);
			return null;
		}
	}

	public RunScenarioResultModel submitScenrioWithParams(Agent agent,
			UUID agentRunId, List<File> paramFiles,
			final RunScenarioModel runScenarioModel) {
		HttpResponse httpResponse = null;
		try {
			final String modelContent = MarshalHelper
					.tryMarshal(runScenarioModel);
			httpResponse = this.httpRequester.postFiles(null,
					buildBaseUrl(agent) + "/test/submitScenarioWithParams/"
							+ agentRunId, "files[]", paramFiles,
					"scenarioModel", new LinkedList<String>() {
						private static final long serialVersionUID = 1L;
						{
							add(modelContent);
						}
					});
			return (RunScenarioResultModel) MarshalHelper.unmarshal(
					RunScenarioResultModel.class, httpResponse.getContent());
		} catch (Exception e) {
			logIt(httpResponse, e);
			return null;
		}
	}

	public RunScenarioResultModel runWithParams(Agent agent, UUID agentRunId) {
		HttpResponse httpResponse = null;
		try {
			if (agent == null || agentRunId == null) {
				return null;
			}
			httpResponse = this.httpRequester.sendPost(buildBaseUrl(agent)
					+ "/test/runWithParams/" + agentRunId.toString(), null,
					null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				logger.error("the response is not valid");
				return null;
			}
			return (RunScenarioResultModel) MarshalHelper.unmarshal(
					RunScenarioResultModel.class, httpResponse.getContent());
		} catch (Exception e) {
			logIt(httpResponse, e);
			return null;
		}
	}

	// there is bug in here
	public TestBriefStatusModel scriptBriefAll(Agent agent, UUID runId) {
		HttpResponse httpResponse = null;
		try {
			if (agent == null || runId == null) {
				return null;
			}
			httpResponse = this.httpRequester.sendGet(buildBaseUrl(agent)
					+ "/test/briefAll/" + runId, null, null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				return null;
			}
			logger.info(httpResponse.getContent());
			return (TestBriefStatusModel) MarshalHelper.unmarshal(
					TestBriefStatusModel.class, httpResponse.getContent());
		} catch (Exception e) {
			logIt(httpResponse, e);
			logger.error(e.toString()
					+ " When get script all brief the agent with hostName "
					+ agent.getHostName());
			FaultTolerenceFactory.getBriefFaultTolerance(agent).doTolerance();
			return null;
		}
	}

	public StopTestModel stop(Agent agent, UUID runId) {
		HttpResponse httpResponse;
		try {
			if (agent == null || runId == null) {
				return null;
			}
			httpResponse = this.httpRequester.sendGet(buildBaseUrl(agent)
					+ "/test/stop/" + runId.toString(), null, null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				return null;
			}
			return (StopTestModel) MarshalHelper.unmarshal(StopTestModel.class,
					httpResponse.getContent());
		} catch (Exception e) {
			logger.error(e.toString() + " when stop the agent with hostName "
					+ agent.getHostName());
			FaultTolerenceFactory.getStopAgentFault(agent, runId).doTolerance();
			return null;
		}
	}

	private String buildBaseUrl(Agent agent) {
		return agent.getHostName() + ":" + agent.getPort();
	}

	public ServerStatusModel getStatus(Agent agent) {
		try {
			HttpResponse httpResponse = this.getHttpRequester().sendGet(
					agent.getHostName() + ":" + agent.getPort() + "/", null,
					null);
			if (HttpRequester.isInvalidResponse(httpResponse)) {
				return null;
			}
			return (ServerStatusModel) MarshalHelper.unmarshal(
					ServerStatusModel.class, httpResponse.getContent());
		} catch (Exception e) {
			logger.error("agent :" + agent.getHostName() + " break down!");
			return null;
		}
	}
}
